  // put your setup code here, to run once:
#include <Time.h>
#include <TimeLib.h>
#include <LiquidCrystal.h>
#include <SPI.h>
#include <FlexiTimer2.h>

  /*******************************************************
    Program: Electrical_Stimulation_Cardiac_Maturation.ino
     Author: Kacey Ronaldson-Bouchard
     Last Updated: 7/30/2018
   *                                                                        *
     This program controls the electrical stimulation of cells. The
     adjustable parameters are: voltage, frequency, and pulse time (the
     duration of time the voltage is high during a given period). It
     generates monophasic waves.
   *                                                                        *
     The parameters are controlled via the serial monitor. To control the
     the parameters, enter "voltage", "frequency", or "pulse time" for the
     respective variable to bring it up, then enter the desired number (no
     decimals). Voltage is between 0 - 5 volts. Frequency is between 0 -
     100 Hz. Pulse time must be less than the period (i.e. 1 / frequency).
   *                                                                        *
     The set up of the circuit involves the use of the Arduino Uno, the
     AD5206 digital potentiometer, and the TLV4110 operational amplifier.
     The SPI library on the arduino is used to control the digital pot via
     the Arduino pins 13 (SCK), 11 (MOSI), and 10 (SS) to pins 8, 7, and 5
     on the AD5206 digital potentiometer respectively. Stimulation is
     achieved by setting the digital potentiometer to a specific resistance
     to achieve the desired voltage to stimulate the cells. The frequency
     is achieved by switching the resistance on the digital potentiometer
     between the set level and the max resistance to obtain a monophasic
     wave. The pulse time is achieved by controlling the when that switch
     occurs.
     LCD programming modified from program from Mark Bramwell, July 2010
   **********************************************************/

  // select the pins used on the LCD panel
  LiquidCrystal lcd(8, 9, 4, 5, 6, 7);

  // define some values used by the panel and buttons
  int lcd_key     = 0;
  int adc_key_in  = 0;
  int levelEstim = 0;
  int level = 0;
  int timeCounter = 0;
  int dayCounter = 0;

  bool firstLoopFlag = 0;
  bool firstDayFlag = 0;
  bool firstHourFlag = 0;
  bool firstMinuteFlag = 0;
  bool boolTest = 0;
  
  float stimFrequency = 2;
  float stimVoltage = 4.5;
  float stimPeriod = 0;
  float pulseWidth = 2;
  float freqIncrement = 0.1;
  float userProgram = 1;

  time_t firstLoopTime = 0;
  time_t secondsSinceStart = 0;
  time_t minutesSinceStart = 0;
  time_t hoursSinceStart = 0;
  time_t daysSinceStart = 0;

#define pulseIncrement 0.1
#define minFreq 0.5
#define maxFreq 6
#define minFreqRamp 0
#define maxFreqRamp 6
#define minVoltage 0
#define maxVoltage 5
#define minPulseWidth 0
#define maxPulseWidth 200
#define SS_PIN 3
#define delayLength 2000
#define potWiperAddress 4 

#define btnRIGHT  0
#define btnUP     1
#define btnDOWN   2
#define btnLEFT   3
#define btnSELECT 4
#define btnNONE   5

  // read the buttons
  int read_LCD_buttons()
  {
    adc_key_in = analogRead(0);      // read the value from the sensor
    // my buttons when read are centered at these valies: 0, 144, 329, 504, 741
    // we add approx 50 to those values and check to see if we are close
    if (adc_key_in > 1000) return btnNONE; // We make this the 1st option for speed reasons since it will be the most likely result
    // For V1.1 us this threshold
    if (adc_key_in < 50)   return btnRIGHT;
    if (adc_key_in < 250)  return btnUP;
    if (adc_key_in < 450)  return btnDOWN;
    if (adc_key_in < 650)  return btnLEFT;
    if (adc_key_in < 850)  return btnSELECT;

    return btnNONE;  // when all others fail, return this...
  }

  float read_value_from_screen(String variableName, String variableUnits, float minVal, float maxVal, float increment)
  {
    float outputVariable = 1.0;
    int minFlag = 0;
    int maxFlag = 0;

    // Set up input screen
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("Set ");
    lcd.print(variableName);
    lcd.setCursor(0, 1);
    lcd.print(outputVariable, 2);
    lcd.print(" ");
    lcd.print(variableUnits);

    lcd_key = read_LCD_buttons();
    while (lcd_key != btnSELECT)
    {
      lcd_key = read_LCD_buttons();
      lcd.setCursor(0, 1);

      switch (lcd_key)
      {
        // Increase Variable
        case btnUP:
          {
            outputVariable += increment;

            // Check If Maximum Is Met
            if (outputVariable >= maxVal)
            {
              outputVariable = maxVal;
              maxFlag = 1;
            }
            else
            {
              maxFlag = 0;
            }

            // Update Display
            delay(150);
            lcd.print("               ");
            lcd.setCursor(0, 1);
            lcd.print(outputVariable, 2);
            lcd.print(" ");
            lcd.print(variableUnits);

            // If At Max Notify User
            if (1 == maxFlag)
            {
              lcd.print("  MAX");
            }

            break;
          }

        // Decrease Variable
        case btnDOWN:
          {
            outputVariable -= increment;

            // Check If Minimum Is Met
            if (outputVariable <= minVal)
            {
              outputVariable = minVal;
              minFlag = 1;
            }
            else
            {
              minFlag = 0;
            }

            // Update Display
            delay(150);
            lcd.setCursor(0, 1);
            lcd.print("               ");
            lcd.setCursor(0, 1);
            lcd.print(outputVariable, 2);
            lcd.print(" ");
            lcd.print(variableUnits);

            // If At Min Notify User
            if (1 == minFlag)
            {
              lcd.print("  MIN");
            }

            break;
          }

        // Variable Is Set
        case btnSELECT:
          {
            lcd.clear();
            lcd.setCursor(0, 0);
            lcd.print(variableName);
            lcd.setCursor(0, 1);
            lcd.print("Set! ");
            lcd.print(outputVariable, 2);
            lcd.print(" ");
            lcd.print(variableUnits);
            delay(delayLength);

            break;
          }
      }
    }

    return outputVariable;
  }

  // allows for changing of the digital potentiometer.
  void digitalPotWrite()
  {
        // take the SS pin low to select the chip:
    digitalWrite(SS_PIN, LOW);
    //  send in the address and value via SPI:
    SPI.transfer(potWiperAddress);
    SPI.transfer(levelEstim);
    digitalWrite(SS_PIN, HIGH);
    
    // Leave on for pulseWidth
    delay(pulseWidth);

    // NEW
    // take the SS pin low to select the chip:
    digitalWrite(SS_PIN, LOW);
    //  send in the address and value via SPI:
    SPI.transfer(potWiperAddress);
    SPI.transfer(0);
    digitalWrite(SS_PIN, HIGH);
  }

  void update_elapsed_time()
  {
    time_t timeNow = 0;
    time_t daysSinceStartTemp = 0;
    
    if (0 == firstLoopFlag)
    {
      firstLoopTime = now();
      firstLoopFlag = 1;
      daysSinceStart = day(firstLoopTime);
    }

    lcd.setCursor(0, 0);
    lcd.print("Freq ");
    lcd.print(stimFrequency, 2);
    lcd.print(" Hz");
    lcd.setCursor(15, 1);
    lcd.print("s");

    // Read time
    timeNow = now() - firstLoopTime;
    secondsSinceStart = second(timeNow);
    minutesSinceStart = minute(timeNow);
    hoursSinceStart = hour(timeNow);
    daysSinceStartTemp = day(timeNow);
    
    // Force to 1 to align with first day
    if (0 == daysSinceStartTemp)
    {
      daysSinceStartTemp = 1;
    }

      //Stim ramp program
      if (1 == userProgram)
      {
        // Check if new Day
        if (daysSinceStartTemp > daysSinceStart)
        {
          dayCounter++;   
          switch (dayCounter) {
            case 0:
              stimFrequency = 2.00;
              break;
            case 1:
              stimFrequency = 2.33;
              break;
            case 2:
              stimFrequency = 2.66;
              break;
            case 3:
              stimFrequency = 3.00;
              break;
            case 4:
              stimFrequency = 3.33;
              break;
            case 5:
              stimFrequency = 3.66;
              break;
            case 6:
              stimFrequency = 4.00;
              break;
            case 7:
              stimFrequency = 4.33;
              break;
            case 8:
              stimFrequency = 4.66;
              break;
            case 9:
              stimFrequency = 5.00;
              break;
            case 10:
              stimFrequency = 5.33;
              break;
            case 11:
              stimFrequency = 5.66;
              break;
            case 12:
              stimFrequency = 6.00;
              break;
            case 13:
              stimFrequency = 6.00;
              break;
            case 14:
              stimFrequency = 6.00;
              break;
            case 15:
              stimFrequency = 2.00;
              break;
            default:
            if (dayCounter < 16)
            {
              stimFrequency = 2.00;
            }
          }
    
          lcd.setCursor(13, 1);
          lcd.print(stimFrequency);
        
          FlexiTimer2::stop();
      
          stimPeriod = 1.0 / stimFrequency * 5000.0;
          FlexiTimer2::set(stimPeriod, 1.0 / 5000, digitalPotWrite);
    
          // Start FlexiTimer
          FlexiTimer2::start();
        }
      }

      // Increment daysSinceStart
      daysSinceStart = daysSinceStartTemp;
    
      // Update Display
      //Seconds
      if (30 > secondsSinceStart)
      {
        lcd.setCursor(14, 1);
        lcd.print(" ");
      }
  
      lcd.setCursor(13, 1);
      lcd.print(secondsSinceStart);
  
      //Minutes
      if (1 <= minutesSinceStart)
      {
        if (0 == firstMinuteFlag)
        {
          lcd.setCursor(9, 1);
          lcd.print("min");
  
          firstMinuteFlag = 1;
        }
  
        if (10 > minutesSinceStart)
        {
          lcd.setCursor(8, 1);
          lcd.print(" ");
        }
        lcd.setCursor(7, 1);
        lcd.print(minutesSinceStart);
      }
  
      //Hours
      if (1 <= hoursSinceStart)
      {
        if (0 == firstHourFlag)
        {
          lcd.setCursor(5, 1);
          lcd.print("h");
  
          firstHourFlag = 1;
        }
  
        if (10 > hoursSinceStart)
        {
          lcd.setCursor(4, 1);
          lcd.print(" ");
        }
        lcd.setCursor(3, 1);
        lcd.print(hoursSinceStart);
      }
  
      //Days
      if (1 < daysSinceStart)
      {
        if (0 == firstDayFlag)
        {
          lcd.setCursor(2, 1);
          lcd.print("d");
        }
        lcd.setCursor(0, 1);
        lcd.print(daysSinceStart);
      }
  }

  void setup()
  {
    // Declare local variables
    int maxFlag = 0;
    int minFlag = 0;

    // Start LCD Library
    lcd.begin(16, 2);

    // Position cursor at top left
    lcd.setCursor(0, 0);

    // Print welcome message
    lcd.print("Initializing...");

    // Set Output mode of pin and set low
    pinMode(SS_PIN, OUTPUT);
    digitalWrite(SS_PIN, LOW);

    // Wait 2 seconds
    delay(delayLength);

    // Initialize SPI:
    SPI.begin();

    // Get Program Selection From User
    userProgram = read_value_from_screen("Program", "", 1, 2, 1);
    
    if (2 == userProgram)
    {
      // Get Frequency from User
      stimFrequency = read_value_from_screen("Start Freq", "Hz", minFreq, maxFreq, freqIncrement);
    }

    // Get Stim Voltage from User
    stimVoltage = read_value_from_screen("Stim Voltage", "V", minVoltage, maxVoltage, freqIncrement);

    // Get Pulse Width from User
    pulseWidth = read_value_from_screen("Pulse Width", "ms", minPulseWidth, maxPulseWidth, freqIncrement);

    // Summarize Settings
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("Start ");
    lcd.print(stimFrequency, 2);
    lcd.print(" Hz");
    lcd.setCursor(0, 1);
    delay(delayLength);

    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("Stim ");
    lcd.print(stimVoltage);
    lcd.print(" V");
    lcd.setCursor(0, 1);
    lcd.print("Pulse ");
    lcd.print(pulseWidth, 2);
    lcd.print(" ms");
    delay(delayLength);

    // Calculate stim voltage value
    levelEstim = (stimVoltage * 255 / 5);

    lcd.clear();

    // Set Up FlexiTimer2
    //Convert Frequency to Period
    stimPeriod = 1.0 / stimFrequency * 5000.0;
    FlexiTimer2::set(stimPeriod, 1.0 / 5000, digitalPotWrite);

    // Ask User to Start Program?
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("Press SELECT");
    lcd.setCursor(0, 1);
    lcd.print("To Start");
    lcd_key = read_LCD_buttons();
    while (lcd_key != btnSELECT)
    {
      delay(100);
      lcd_key = read_LCD_buttons();
    }

    // Start FlexiTimer
    FlexiTimer2::start();

    // Clear LCD
    lcd.clear();
  }


void loop()
{
  update_elapsed_time();
}

